#ifndef __MULTI_STAGE_H
#define __MULTI_STAGE_H

#include "orbitersdk.h"
#include <math.h>

const double PI = 3.14159265359;
const double RAD = PI/180.0;

double GetCurrentValue(VESSEL *vessel, double sl, double vac)
{
	double c_atm=0;
	double c_value=0;
	c_atm = vessel->GetAtmPressure() / 101.4e3; 
	c_value = c_atm * (vac-sl);				
	c_value += sl;								
	return c_value;
}

void StageTransform(VESSEL *vessel, VESSELSTATUS *vs, VECTOR3 ofs, VECTOR3 vel)
{
	VECTOR3 rofs, rvel = {vs->rvel.x, vs->rvel.y, vs->rvel.z};
// Staging Velocity represents 
// Need to do some transforms to get the correct vector to eject the stage
	vessel->Local2Rel (ofs, vs->rpos);
	vessel->GlobalRot (vel, rofs);
	vs->rvel.x = rvel.x+rofs.x;
	vs->rvel.y = rvel.y+rofs.y;
	vs->rvel.z = rvel.z+rofs.z;
}

// ==========================================================
// Interface for derived vessel class: Atlantis
// ==========================================================

class IStage
{
public:
	virtual void Init ();
	virtual void Exit (VESSEL *vessel);
	virtual void Setup (VESSEL *vessel, double stage_sep_time);
	virtual void LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs);
	virtual void SaveState (VESSEL *vessel, FILEHANDLE scn);
	virtual void Timestep (VESSEL *vessel, double simt);
	virtual void Animate (VESSEL *vessel, double simt);
	virtual void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	virtual void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	virtual int ConsumeKey (VESSEL *vessel, const char *keystate);
};

void IStage::Init ()
{
};
void IStage::Exit (VESSEL *vessel)
{
};
void IStage::Setup (VESSEL *vessel, double stage_sep_time)
{
};
void IStage::LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{
};
void IStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
};
void IStage::Timestep (VESSEL *vessel, double simt)
{
};
void IStage::Animate (VESSEL *vessel, double simt)
{
};
void IStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void IStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int IStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	return 0;
};

class Multi_Stage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void SetClassCaps (VESSEL *vessel, FILEHANDLE cfg);
	void LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
	void SetStage (IStage *Stage, VESSEL *vessel, double simt);
private:
	IStage *cStage;
};


void Multi_Stage::Init ()
{
	if (cStage)
		cStage->Init();
};
void Multi_Stage::Exit (VESSEL *vessel)
{
	if (cStage)
		cStage->Exit(vessel);
};
void Multi_Stage::SetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	if (cStage)
		cStage->Init();
};
void Multi_Stage::LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{
	if (cStage)
		cStage->LoadState(vessel,scn,def_vs);
};
void Multi_Stage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	if (cStage)
		cStage->SaveState(vessel,scn);
};
void Multi_Stage::Timestep (VESSEL *vessel, double simt)
{
	if (cStage)
		cStage->Timestep(vessel,simt);
};
void Multi_Stage::Animate (VESSEL *vessel, double simt)
{
	if (cStage)
		cStage->Animate(vessel,simt);
};
void Multi_Stage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
	if (cStage)
		cStage->VisualCreated(vessel,vis,refcount);
};
void Multi_Stage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
	if (cStage)
		cStage->VisualDestroyed(vessel,vis,refcount);
};
int Multi_Stage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (cStage)
		return cStage->ConsumeKey(vessel,keystate);
	return 0;
};
void Multi_Stage::SetStage (IStage *Stage, VESSEL *vessel, double simt)
{
	double *booster;
	cStage=Stage;
	if (cStage)
	{
		cStage->Init();
		cStage->Setup(vessel,simt);
		booster = vessel->GetMainThrustModPtr();
		*booster = 0;

	};
};


Multi_Stage *Stack;
#endif // !__MULTI_STAGE_H

