// ==============================================================
// ORBITER vessel module: Redstone
// ==============================================================

#include "orbitersdk.h"

const double PI  = 3.14159265359;
const double RAD = PI/180.0;
const VECTOR3 OFS_AB = { 0.0, 0.0, -1.9};
bool bManualSeparate = false;
double stage_sep;
UINT stage = 0;


void SetAbortStage (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (1.9);
	vessel->SetCOG_elev (1.9);
	vessel->SetEmptyMass (145.0);
	vessel->SetMaxFuelMass (8.0);
	vessel->SetISP (85*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 3800.47);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
//	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_AB.z-26.0), _V(-.7,.7,OFS_AB.z-26.0), _V(.7,.7,OFS_AB.z-26.0));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(1.36,1.37,.04));
	vessel->SetCrossSections (_V(0.81,0.93,0.37));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, 1.4, 1.4);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("merc_spike");
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref; 
	m_exhaust_pos= _V(.052647,0.031591,-0.008362);
	m_exhaust_ref = _V(.282,.103,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);
	m_exhaust_pos= _V(-0.006861,-0.060231,-0.009827);
	m_exhaust_ref = _V(0,-.300,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);
	m_exhaust_pos= _V(-0.062828,0.031106,-0.009827);
	m_exhaust_ref = _V(-.282,.103,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);

}


// ==============================================================
// API interface
// ==============================================================



// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	stage = 0;
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}

DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				vessel->SetFuelMass(8.0);
			return 1;
		}
	}
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	//SetAbortStage(vessel);
	SetAbortStage(vessel);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{

}