#ifndef __ATLAS_H
#define __ATLAS_H
#define N	16	// flight plan table size

#include "orbitersdk.h"
#include "multi_stage.h"
#include "..\mercury_capsule.h"
const VECTOR3 OFS_ATLAS = { 0.0, 0.0, -11.4-0.96-1.64};
const VECTOR3 OFS_BOOSTER = { 0.0, 0.0, -22.8-0.96-1.64+1.93};
const VECTOR3 OFS_ADAPTER = { 0.0, 0.0, -0.96-.82};
double const slBooster=667234.0;
double const vacBooster=758711.0;
double const slSub=252717.0;
double const vacSub=363218.0;
double const boosterISP=286;
double const vacISP=303;


// added by vinka
const double met[N]=   {0.  ,5.,15.,35.,55.,75.,95.,115.,135.,155.,165.,215.,225.,290.,300.,306.}; // MET in sec
const double cpitch[N]={90.,90.,80.,70.,60.,50.,40.,30. ,20. ,10. ,0.  ,0.  ,-13. ,-13. ,0.  ,0.};	// Commanded pitch in 
//double met[N];
//double cpitch[N];

static double GetCPitch(double t);
static void AutoPilot(double t);
static char current_key=0;
// read current pitch order in table
static double GetCPitch(double t)
{
 int i;
 if (t>met[N-1]) return cpitch[N-1];
 i=1;
 while (met[i]<t) i++;
 return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

static void AutoPilot(double t)
{
 static int first_time=1;
 static int out_level=0;
 double level=0.;
 double altitude;
 double pitch;
 double pitch_c;

 // suggestion : you could also check here for minimum eccentricity with the new API
 // and cut off when reached.

 if (t>(met[N-1]-1.))	// stop any rotation 1 sec before SECO
	{
	if (first_time) 
		{
		current_key = OAPI_KEY_NUMPAD5;
		first_time=0;
		}
	else
		current_key=0;
    return;
	}

 // navigation
 oapiGetFocusPitch(&pitch);
 pitch=pitch*180./PI;
 oapiGetFocusAltitude(&altitude);

 // guidance
 pitch_c=GetCPitch(t);

 // control
 if (t<5.)
	current_key=0;	// inhibit pitch at launch
 else
	{
	level = pitch_c - pitch;
	if (altitude<50.e3)		// due to atmosphere we constantly have to force correction
		{
		if (level>1.) current_key = OAPI_KEY_NUMPAD2;
		else if (level<-1.) current_key = OAPI_KEY_NUMPAD8;
		else current_key = OAPI_KEY_NUMPAD5;
		}
	else
		{
		if (fabs(level)<1.)	// above atmosphere, soft correction
			{
			current_key = OAPI_KEY_NUMPAD5;
			out_level = 0;
			}
		else
			{
			if (out_level<(int)(2.*level))
				{
				out_level++;
				current_key = OAPI_KEY_NUMPAD2;
				}
			else if (out_level>(int)(2.*level))
				{
				current_key = OAPI_KEY_NUMPAD8;
				out_level--;
				}
			else
				current_key = 0;
			}

		}	
	}
}
// end of added by vinka



void AddAtlas(VESSEL *vessel)
{
	VECTOR3 mesh_dir;
	mesh_dir=OFS_ADAPTER;
	vessel->AddMesh ("atl_adapt", &mesh_dir);
	mesh_dir=OFS_ATLAS;
	vessel->AddMesh ("atl_main", &mesh_dir);
};


void AddBooster(VESSEL *vessel)
{
	VECTOR3 mesh_dir;
	mesh_dir=OFS_BOOSTER;
	vessel->AddMesh ("atl_boost", &mesh_dir);
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(0,0,-22.6-1.1-1.64);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 4.0, 0.5, &m_exhaust_ref);
	m_exhaust_pos= _V(-1.5,0,-22.6-1.1-1.64);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 4.0, 0.5, &m_exhaust_ref);
	m_exhaust_pos= _V(1.5,0,-22.6-1.1-1.64);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 4.0, 0.5, &m_exhaust_ref);

	m_exhaust_pos= _V(0,0,-17.7-1.1-1.64);
	m_exhaust_ref = _V(0,.866,-.5);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 5.8, 0.075, &m_exhaust_ref);
	m_exhaust_pos= _V(0,0,-17.7-1.1-1.64);
	m_exhaust_ref = _V(0,-.866,-.5);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 5.8, 0.075, &m_exhaust_ref);
};

void AddSubstainer(VESSEL *vessel)
{
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(0,0,-22.6-1.1-1.64);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 4.0, 0.5, &m_exhaust_ref);

	m_exhaust_pos= _V(0,0,-17.7-1.1-1.64);
	m_exhaust_ref = _V(0,.866,-.5);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 5.8, 0.075, &m_exhaust_ref);
	m_exhaust_pos= _V(0,0,-17.7-1.1-1.64);
	m_exhaust_ref = _V(0,-.866,-.5);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 5.8, 0.075, &m_exhaust_ref);

};



class AtlasSubstainer:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetAtlasSubstainer(VESSEL *vessel);
	void SetAtlasSubstainerNoAbort(VESSEL *vessel);
	void JettisonMercury(VESSEL *vessel);
	bool bAbortJettisoned;
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
};


void AtlasSubstainer::Init ()
{
	bAbortJettisoned=false;
	bManualSeparate = false;
	bAbort=false;
	doAbort=false;
};
void AtlasSubstainer::Exit (VESSEL *vessel)
{
};
void AtlasSubstainer::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetAtlasSubstainer(vessel);
};
void AtlasSubstainer::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	double doAbort=0.0;
    if (!strnicmp (line, "ABORT_JETT", 10)) {
        sscanf (line+10, "%lf", &doAbort);
		if (doAbort == 1)
		{
			bAbortJettisoned=true;
			SetAtlasSubstainerNoAbort(vessel);
		}
    } else {
        vessel->ParseScenarioLine (line, def_vs);
    }
};
void AtlasSubstainer::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 1);
	oapiWriteScenario_float (scn, "ABORT_JETT", bAbortJettisoned);	
};
void AtlasSubstainer::Timestep (VESSEL *vessel, double simt)
{
	IStage *newStage;
	if (!bAbortJettisoned)
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			_JettisonAbortTower(vessel);
			SetAtlasSubstainerNoAbort(vessel);
			bManualSeparate=false;
			bAbortJettisoned=true;
		} 
		else if (bAbort)
		{
			vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
			bAbort = false;
			doAbort = true;
		}
		else if (doAbort)
		{
			JettisonMercury(vessel);
			newStage = new Mercury_Abort();
			Stack->SetStage(newStage,vessel,simt);
			return;
		}
	}
	else 
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			bManualSeparate=false;
			JettisonMercury(vessel);
			newStage = new Mercury_WithRetro();
			Stack->SetStage(newStage,vessel,simt);
			return;
		}
	}
	DoThrustAdjust(vessel,slSub,vacSub);
	// added by vinka
	//AutoPilot(simt-ignition_time);
};
void AtlasSubstainer::Animate (VESSEL *vessel, double simt)
{
};
void AtlasSubstainer::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void AtlasSubstainer::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int AtlasSubstainer::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	// added by vinka
	// if no key is to be processed, send a control key
	/*if (current_key!=0)
		{
		char *keys = (char *)&keystate[0];
		keys[current_key]|=0x80;
		}*/
	return 0;
};

void AtlasSubstainer::SetAtlasSubstainer(VESSEL *vessel)
{
// ==============================================================
	double *booster_thrust;
	booster_thrust = vessel->GetMainThrustModPtr();
	*booster_thrust = 0.0;	
	vessel->SetSize (11.0+0.82+1.2+1.9);
	vessel->SetCOG_elev (11.0+0.82+1.2);
	vessel->SetEmptyMass(2347+1360.0);
	vessel->SetMaxFuelMass (113050-2347);
	vessel->SetISP (vacISP*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, vacSub);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);		
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(39.7,39.7,1.45));
	vessel->SetCrossSections (_V(80.15,85.27,10.89));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, MERC_DRAG, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAbortTower(vessel);
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddAtlas(vessel);
	AddSubstainer(vessel);
};

void AtlasSubstainer::SetAtlasSubstainerNoAbort(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (11.0+0.82+1.2);
	vessel->SetCOG_elev (11.0+0.82+1.2);
	vessel->SetEmptyMass (2347+1360);
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(39.7,39.7,1.45));
	vessel->SetCrossSections (_V(80.15,85.27,10.89));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, MERC_DRAG, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddAtlas(vessel);
	AddSubstainer(vessel);
};

void AtlasSubstainer::JettisonMercury(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_ATLAS;
	vel = _V(0,0,-0.5);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Atlas");
	VESSEL::Create (name, "Atlas_Substainer_Rocket", vs);
};


class AtlasBooster:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetAtlasBooster(VESSEL *vessel);
	void JettisonMercury(VESSEL *vessel);
	void JettisonBooster(VESSEL *vessel);
	void doBoosterTimeStep (VESSEL *vessel, double simt);
	bool bManualSeparate;
	double last_simt;
	bool bAbort;
	bool doAbort;
	double ignition_time;
	double bEngineIgnited;
};

void AtlasBooster::Init ()
{
	bManualSeparate = false;
	bAbort=false;
	doAbort=false;
	ignition_time=0.0;
	last_simt=0;
	bEngineIgnited = false;
/*	met[0]=0;
	cpitch[0]=90;
	met[1]=16;
	cpitch[1]=90;
	for (int i=2; i<N-1;i++)
	{
		met[i]=met[i-1]+1;
		cpitch[i]=cpitch[i-1]-0.5;
	};
	met[N-1]=360;
	cpitch[N-1]=0;
*/
};
void AtlasBooster::Exit (VESSEL *vessel)
{
};
void AtlasBooster::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetAtlasBooster(vessel);
};
void AtlasBooster::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	double old_ignition_time=0.0;
    if (!strnicmp (line, "ITIME", 5)) {
        sscanf (line+5, "%lf", &old_ignition_time);
		ignition_time=-old_ignition_time;
    } else if (!strnicmp (line, "ENGINE_IGNITED", 14)) {
        sscanf (line+14, "%lf", &bEngineIgnited);
    } else {
        vessel->ParseScenarioLine (line, def_vs);
    }
};
void AtlasBooster::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 0);
	oapiWriteScenario_float (scn, "ITIME", last_simt-ignition_time);
	oapiWriteScenario_float (scn, "ENGINE_IGNITED", bEngineIgnited);
};
void AtlasBooster::Timestep (VESSEL *vessel, double simt)
{

	IStage *newStage;
	if (bAbort)
	{
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);
		doBoosterTimeStep(vessel,simt);	
		bAbort = false;
		doAbort = true;
	}
	else if (doAbort)
	{
		JettisonMercury(vessel);
		newStage = new Mercury_Abort();
		Stack->SetStage(newStage,vessel,simt);
		return;
	};	

	if (bEngineIgnited)
	{
		if (simt-ignition_time > 135 || bManualSeparate)
		{
			JettisonBooster(vessel);
			newStage = new AtlasSubstainer();
			Stack->SetStage(newStage,vessel,simt);
			vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
		}
		else 
		{
			doBoosterTimeStep(vessel,simt);	
		};
	}
	else
	{
		if (vessel->GetEngineLevel(ENGINE_MAIN) > .95)
		{
			ignition_time=simt;
			bEngineIgnited=true;
		};
	};

	// added by vinka
	//AutoPilot(simt-ignition_time);		
	last_simt=simt;

};
void AtlasBooster::Animate (VESSEL *vessel, double simt)
{
};
void AtlasBooster::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void AtlasBooster::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int AtlasBooster::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	// added by vinka
	// if no key is to be processed, send a control key
	/*if (current_key!=0)
		{
		char *keys = (char *)&keystate[0];
		keys[current_key]|=0x80;
		}*/
	return 0;
};

void AtlasBooster::SetAtlasBooster(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (11.0+0.82+1.2+1.9);
	vessel->SetCOG_elev (11.0+0.82+1.2);
	vessel->SetEmptyMass (2347+3050+1360);
	vessel->SetMaxFuelMass (113050-2347);
	vessel->SetFuelMass(113050-2347);
	vessel->SetISP (vacISP*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, vacSub);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);		
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_ATLAS.z-11.0), _V(-.7,.7,OFS_ATLAS.z-11.0), _V(.7,.7,OFS_ATLAS.z-11.0));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(39.7,39.7,1.45));
	vessel->SetCrossSections (_V(80.15,85.27,10.89));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, MERC_DRAG, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAbortTower(vessel);
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddAtlas(vessel);
	AddBooster(vessel);
};


void AtlasBooster::JettisonMercury(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_ATLAS;
	vel = _V(0,0,-0.5);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Atlas");
	VESSEL::Create (name, "Atlas_Booster_Rocket", vs);
};

void AtlasBooster::JettisonBooster(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_BOOSTER;
	vel = _V(0,0,-0.5);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Booster");
	VESSEL::Create (name, "Atlas_Booster_Pack", vs);
};

void AtlasBooster::doBoosterTimeStep (VESSEL *vessel, double simt)
{
	double *booster_thrust;
	double c_booster_thrust;
	double fuel_consumed;
	booster_thrust = vessel->GetMainThrustModPtr();
	*booster_thrust = 0.0;
	fuel_consumed = ((simt-last_simt)*vacBooster) /(boosterISP*9.8);
	vessel->SetFuelMass(vessel->GetFuelMass()-fuel_consumed*2);
	c_booster_thrust = GetCurrentValue(vessel,slBooster,vacBooster);
	*booster_thrust = (GetCurrentValue(vessel,slSub,vacSub)-GetCurrentValue(vessel,vacSub,vacSub))+ (c_booster_thrust * 2);
}

#endif 
