// ==============================================================
// ORBITER vessel module: Atlas Substainer
// ==============================================================

#include "orbitersdk.h"
const VECTOR3 OFS_ATLAS = {0.0, 0.0, 0.0};
const VECTOR3 OFS_ADAPTER = { 0.0, 0.0, 11.4+.82};


// ==============================================================
// API interface
// ==============================================================



// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}


// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	vessel->SetSize (11.4+.84);
	vessel->SetCOG_elev (11.4+.84);
	vessel->SetEmptyMass (2347);
	vessel->SetMaxThrust (ENGINE_MAIN, 0);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
	vessel->SetPMI (_V(35.7,35.7,1.45));
	vessel->SetCrossSections (_V(73.15,78.27,10.89));
	vessel->SetCW (0.1, 0.1, 0.1, 0.1);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	mesh_dir=OFS_ADAPTER;
	vessel->AddMesh ("atl_adapt", &mesh_dir);
	mesh_dir=OFS_ATLAS;
	vessel->AddMesh ("atl_main", &mesh_dir);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{

}

