// ==============================================================
// ORBITER vessel module: Mercury Redstone
// ==============================================================

#include "orbitersdk.h"
#include "multi_stage.h"
#include "..\mercury_capsule.h"
#include "redstone.h"

void SetConfiguration(int status, VESSEL *vessel)
{
	IStage *newStage;
	switch (status) {
	case 0:
		newStage = new Redstone();
		break;
	case 3:
		newStage = new Mercury_Abort();
		break;
	case 4:
		newStage = new Mercury_WithRetro();
		break;
	case 5:
		newStage = new Mercury_Capsule();
		break;
	}
	Stack->SetStage(newStage,vessel,0);
};

Redstone *Test;

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	Stack = new Multi_Stage();
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (Stack) delete Stack;
	if (vessel) delete vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	Stack->ConsumeKey(vessel,keystate);
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	IStage *newStage;
	newStage = new Redstone();
	Stack->SetStage(newStage,vessel,0);
//	newStage->Setup(vessel,0);
//	Test = new Redstone();
//	Test->Setup(vessel,0);

}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	Stack->Timestep(vessel,simt);
}

DLLCLBK void ovcLoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{
    char *line;
	int status = 0;
	double test=0.0;
	bool configurationset = false;
	while (oapiReadScenario_nextline (scn, line)) {
		if (configurationset)
		{
			Stack->ParseLine(vessel,line,def_vs);
		}
		else
		{
			if (!strnicmp (line, "CONFIGURATION", 13)) {
				sscanf (line+13, "%d", &status);
				SetConfiguration(status, vessel);
				configurationset=true;
			} else {
				vessel->ParseScenarioLine (line, def_vs);
			}
		}
    }
}

// Save status to scenario file
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	vessel->SaveDefaultState(scn);
	Stack->SaveState(vessel,scn);
}

