#ifndef __REDSTONE_H
#define __REDSTONE_H

#include "orbitersdk.h"
#include "multi_stage.h"
#include "..\mercury_capsule.h"
const VECTOR3 OFS_RED = { 0.0, 0.0, -8.8-.96};
const double slThrust = 367455;
const double vacThrust = 414340;

#endif 

void AddRedstone(VESSEL *vessel)
{
	VECTOR3 mesh_dir=OFS_RED;
	vessel->AddMesh ("Redstone", &mesh_dir);
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(0,0,OFS_RED.z-8.8);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 4.0, 0.5, &m_exhaust_ref);
};

class Redstone:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetRedstone(VESSEL *vessel);
	void SetRedstoneNoAbort(VESSEL *vessel);
	void JettisonMercury(VESSEL *vessel);
	void JettisonAbortTower(VESSEL *vessel);
	bool bAbortJettisoned;
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
};


void Redstone::Init ()
{
	bAbortJettisoned=false;
	bManualSeparate = false;
	bAbort=false;
	doAbort=false;
};
void Redstone::Exit (VESSEL *vessel)
{
};
void Redstone::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetRedstone(vessel);
};
void Redstone::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	double doAbort=0.0;
    if (!strnicmp (line, "ABORT_JETT", 10)) {
        sscanf (line+10, "%lf", &doAbort);
		if (doAbort == 1)
		{
			bAbortJettisoned=true;
			SetRedstoneNoAbort(vessel);
		};
    } else {
        vessel->ParseScenarioLine (line, def_vs);
    }

};
void Redstone::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 0);
	oapiWriteScenario_float (scn, "ABORT_JETT", bAbortJettisoned);	
};
void Redstone::Timestep (VESSEL *vessel, double simt)
{
	IStage *newStage;
	double *booster_thrust;
	booster_thrust = vessel->GetMainThrustModPtr();
	if (!bAbortJettisoned)
	{

		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			JettisonAbortTower(vessel);
			SetRedstoneNoAbort(vessel);
			bManualSeparate=false;
			bAbortJettisoned=true;
		} 
		else if (bAbort)
		{
			vessel->SetEngineLevel(ENGINE_MAIN,1.0);	
			bAbort = false;
			doAbort = true;
		}
		else if (doAbort)
		{
			JettisonMercury(vessel);
			newStage = new Mercury_Abort();
			Stack->SetStage(newStage,vessel,simt);
			return;
		}
	}
	else 
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			bManualSeparate=false;
			JettisonMercury(vessel);
			newStage = new Mercury_WithRetro();
			Stack->SetStage(newStage,vessel,simt);
			return;
		}
	}
	*booster_thrust = 0;
	*booster_thrust = (GetCurrentValue(vessel,slThrust,vacThrust)-vacThrust);
	vessel->SetMaxThrust(ENGINE_HOVER,(GetCurrentValue(vessel,slThrust,vacThrust)-vacThrust));
};
void Redstone::Animate (VESSEL *vessel, double simt)
{
};
void Redstone::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void Redstone::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Redstone::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	return 0;
};

void Redstone::SetRedstone(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (8.8+1.5+1.9);
	vessel->SetCOG_elev (8.8+1.5);
	vessel->SetEmptyMass (3890.0);
	vessel->SetMaxFuelMass (28430.0-3890.0);
	vessel->SetFuelMass(28430.0-3890.0);
	vessel->SetISP (235.0*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_RED.z-8.2), _V(-.7,.7,OFS_RED.z-8.2), _V(.7,.7,OFS_RED.z-8.2));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(33.0,33.0,0.51));
	vessel->SetCrossSections (_V(38.4,38.4,4.04));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, MERC_DRAG, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAbortTower(vessel);
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddRedstone(vessel);
};

void Redstone::SetRedstoneNoAbort(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (8.8+1.5);
	vessel->SetCOG_elev (8.8+1.5);
	vessel->SetEmptyMass (3890.0);
	vessel->SetMaxFuelMass (28430.0-3890.0);
	vessel->SetFuelMass(vessel->GetFuelMass());
	vessel->SetISP (235.0*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 369024.0);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
	vessel->SetEngineLevel(ENGINE_MAIN,vessel->GetEngineLevel(ENGINE_MAIN));
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,-2.5), _V(-.7,.7,-2.5), _V(.7,.7,-2.5));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(31.3,31.3,0.51));
	vessel->SetCrossSections (_V(37.5,37.5,4.04));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.3, MERC_DRAG, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddRedstone(vessel);
};

void Redstone::JettisonAbortTower(VESSEL *vessel)
{
	_JettisonAbortTower(vessel);
};

void Redstone::JettisonMercury(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_RED;
	vel = _V(0,0,-0.5);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Redstone");
	VESSEL::Create (name, "Redstone", vs);
};