#ifndef __MERCURY_CAPSULE_H
#define __MERCURY_CAPSULE_H

#include "orbitersdk.h"
#include "multi_stage.h"
const double MERC_DRAG=0.75;
const VECTOR3 OFS_RETROPACK = { 0.0, 0.0, -0.9-.11};
const VECTOR3 OFS_ABORTTOWER = { 0.0, 0.0, 1.00+.48+1.35};
const VECTOR3 OFS_ANTENNAHOUSING = { 0.0, 0.0, 1.00+.24};
const VECTOR3 OFS_CAPSULE = { 0.0, 0.0, 0.0};
const VECTOR3 OFS_MAINCHUTE = { 0.0, 0.0, 1.0};
const VECTOR3 OFS_LANDINGBAG = { 0.0, 0.0, -0.85-.34};
const VECTOR3 OFS_BEACON = { 0.0, 0.425, -0.9-.15};

void _JettisonAbortTower(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_ABORTTOWER;		
	vel = _V(0,1,0);	
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	
	vs.vrot.y = 0.0;	
	vs.vrot.z = 0.0;	
	vs.eng_main=1.0;
	vs.fuel=1.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Abort");
	VESSEL::Create (name, "Mercury_Abort", vs);
};

void _JettisonRetro(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_RETROPACK;
	vel = _V(0,0,-0.5);
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Retro");
	VESSEL::Create (name, "Mercury_Retro", vs);
}

void _JettisonDrogue(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_ANTENNAHOUSING;		
	vel = _V(0,1,0);	
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	
	vs.vrot.y = 0.0;	
	vs.vrot.z = 0.0;	
	vs.eng_main=1.0;
	vs.fuel=1.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Drogue");
	VESSEL::Create (name, "Mercury_Drogue", vs);
};

void _JettisonMain(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_MAINCHUTE;		
	vel = _V(0,1,0);	
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	
	vs.vrot.y = 0.0;	
	vs.vrot.z = 0.0;	
	vs.eng_main=1.0;
	vs.fuel=1.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Main");
	VESSEL::Create (name, "Mercury_Chute", vs);
};


void _JettisonAntenna(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_ANTENNAHOUSING;		
	vel = _V(0,1,0);	
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	
	vs.vrot.y = 0.0;	
	vs.vrot.z = 0.0;	
	vs.eng_main=1.0;
	vs.fuel=1.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Antenna");
	VESSEL::Create (name, "Mercury_Antenna", vs);
};

void _JettisonBeacon(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_BEACON;		
	vel = _V(0,.1,0);	
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	
	vs.vrot.y = 0.0;	
	vs.vrot.z = 0.0;	
	vs.eng_main=1.0;
	vs.fuel=1.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Beacon");
	VESSEL::Create (name, "Mercury_Beacon", vs);
};

void AddCapsule(VESSEL *vessel)
{
	VECTOR3 mesh_dir=_V(0,0,0);
	vessel->AddMesh ("merc_capsule", &mesh_dir);
};

void AddAntennaHousing(VESSEL *vessel)
{
	VECTOR3 mesh_dir=OFS_ANTENNAHOUSING;
	vessel->AddMesh ("merc_anthouse", &mesh_dir);
};

void AddRetro(VESSEL *vessel)
{
	VECTOR3 mesh_dir=OFS_RETROPACK;
	vessel->AddMesh ("merc_retro", &mesh_dir);
};

void AddAbortTower(VESSEL *vessel)
{
	VECTOR3 mesh_dir=OFS_ABORTTOWER;
	vessel->AddMesh ("merc_spike", &mesh_dir);
};

void AddLandingBag(VESSEL *vessel)
{
	VECTOR3 mesh_dir=OFS_LANDINGBAG;
	vessel->AddMesh ("merc_bag", &mesh_dir);
};

void AddBeacon(VESSEL *vessel)
{
	VECTOR3 mesh_dir=OFS_BEACON;
	vessel->AddMesh ("merc_beacon", &mesh_dir);
};


void AddAttitudeJets(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	att_ref = _V(0,-.35,.8);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);
	att_ref = _V(0,.35,.8);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	att_ref = _V(-.35,0,.8);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	att_ref = _V(.35,0,.8);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	att_ref = _V(0,-1.0,-.8);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);
	att_ref = _V(0,1.0,-.8);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);
	att_ref = _V(0,1.0,-.8);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);
	att_ref = _V(0,-1.0,-.8);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.2,0.2);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);
}

void SetCapsuleWithRetro (VESSEL *vessel, bool withBeacon)
{
// ==============================================================
	vessel->SetSize (1.5);
	vessel->SetCOG_elev (1.5);
	vessel->SetEmptyMass (1355.0);
	vessel->SetMaxFuelMass (15.0);
	vessel->SetFuelMass(9.9);
	vessel->SetISP (51*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 1650.0*3);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e1);
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,-1.0), _V(-.7,.7,-1.0), _V(.7,.7,-1.0));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(0.85,0.83,.5));
	vessel->SetCrossSections (_V(3.5,3.37,3.81));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
// Setup Parachutes
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddRetro(vessel);
	AddAttitudeJets(vessel);
	if (withBeacon)
		AddBeacon(vessel);
}


class Mercury_Capsule:public IStage
{
public:
	Mercury_Capsule();
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetCapsule(VESSEL *vessel);
	void SetCapsuleMain(VESSEL *vessel);
	void DeployDrogue(VESSEL *vessel);
	void DeployMain(VESSEL *vessel);
	void SepParachute(VESSEL *vessel);
	bool bManualSeparate;
	bool bDrogueDeployed;
	bool bMainDeployed;
	bool bParachute;
	bool bParachuteJettisoned;
	bool bDrogueJettisoned;
	bool bMainJettisoned;
	bool bBackupDeployed;
	bool bBackupJettisoned;
	double stage_sep;
	double PressDelta;
	double LastPress;
};

Mercury_Capsule::Mercury_Capsule()
{
	Init();
};
void Mercury_Capsule::Init ()
{
	bManualSeparate = false;
	bDrogueDeployed = false;
	bMainDeployed = false;
	bParachute = false;
	bDrogueJettisoned=false;
	bMainJettisoned=false;
	bBackupJettisoned=false;
	bBackupDeployed=false;
	bParachuteJettisoned=false;
	stage_sep = 0.0;
	PressDelta=0.0;
	LastPress=0.0;
}

void Mercury_Capsule::Exit(VESSEL *vessel)
{
}
void Mercury_Capsule::Setup (VESSEL *vessel, double stage_sep_time)
{
	stage_sep=stage_sep_time;
	if (vessel->GetAtmPressure()>66000)
	{
		bDrogueJettisoned=true;
		bDrogueDeployed=true;
		SetCapsuleMain(vessel);
	}
	else
		SetCapsule(vessel);
	LastPress=vessel->GetAtmPressure();
}

void Mercury_Capsule::LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{

}

void Mercury_Capsule::SaveState (VESSEL *vessel, FILEHANDLE scn)
{

}

void Mercury_Capsule::Timestep (VESSEL *vessel, double simt)
{
	PressDelta = LastPress-vessel->GetAtmPressure();
	LastPress = vessel->GetAtmPressure();

	if (vessel->GetAtmPressure() > 45000 && !bDrogueDeployed && PressDelta < 0)
	{
		bDrogueDeployed = true;
		DeployDrogue(vessel);
	};

	if (vessel->GetAtmPressure() > 66000 && !bDrogueJettisoned && PressDelta < 0)
	{
		bDrogueJettisoned=true;
		_JettisonDrogue(vessel);
		SetCapsuleMain(vessel);
	};			

	if (vessel->GetAtmPressure() > 68000 && !bMainDeployed && PressDelta < 0)
	{
		bMainDeployed=true;
		DeployMain(vessel);
	};			

	if (bParachute && !bBackupDeployed && bMainJettisoned && PressDelta < 0)
	{
		bBackupDeployed=true;
		DeployMain(vessel);
	};			
	
	
	if (bParachuteJettisoned && (bMainDeployed || bBackupDeployed || bDrogueDeployed)) 
	{
		bParachuteJettisoned=false;
		SepParachute(vessel);
	};
	
}

void Mercury_Capsule::Animate (VESSEL *vessel, double simt)
{

}

void Mercury_Capsule::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{

}

void Mercury_Capsule::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)

{

}

int Mercury_Capsule::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0;
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
	}
	else 
	{ 
		if (KEYDOWN (keystate, OAPI_KEY_J)) { //Jettison Parachutes
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bParachuteJettisoned = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_P)) { // "Parachutes"
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0))
				bParachute = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_DIVIDE)) { // "Disable Laterals"
			oapiSetAttitudeMode(vessel->GetHandle(),0);
			return 0;
		}
	}
	return 0;
};

void Mercury_Capsule::SetCapsuleMain(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (1.5);
	vessel->SetCOG_elev (1.5);
	vessel->SetEmptyMass (1130.0);
	vessel->SetMaxFuelMass (0.1);
	vessel->SetFuelMass(0.0);
	vessel->SetISP (0);
	vessel->SetMaxThrust (ENGINE_MAIN, 0);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e1);
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,-.8), _V(-.7,.7,-.8), _V(.7,.7,-.8));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(0.66,0.66,.42));
	vessel->SetCrossSections (_V(2.8,2.8,3.27));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
// Setup Parachutes
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	AddCapsule(vessel);
	if (bMainDeployed)
		AddLandingBag(vessel);
	AddAttitudeJets(vessel);	
}



void Mercury_Capsule::SetCapsule(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (1.5);
	vessel->SetCOG_elev (1.5);
	vessel->SetEmptyMass (1130.0);
	vessel->SetMaxFuelMass (0.1);
	vessel->SetFuelMass(0.0);
	vessel->SetISP (0);
	vessel->SetMaxThrust (ENGINE_MAIN, 0);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e1);
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,-1.5), _V(-.7,.7,-1.5), _V(.7,.7,-1.5));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(0.66,0.66,.42));
	vessel->SetCrossSections (_V(2.8,2.8,3.27));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
// Setup Parachutes
	if (vessel->GetAtmPressure() > 66000)
	{
		bDrogueJettisoned=true;
		bDrogueDeployed=true;
	};
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	AddCapsule(vessel);
	AddAntennaHousing(vessel);
	AddAttitudeJets(vessel);	
}

void Mercury_Capsule::DeployDrogue(VESSEL *vessel)
{
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetCrossSections (_V(5.8,5.8,3.5));
	vessel->SetCW (0.1, 0.1, 0.1, 10.0);
// ==============================================================
	vessel->SetSize (9);
	vessel->ClearMeshes();
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	VECTOR3 mesh_dir=_V(0,0,1.0+.48);
	vessel->AddMesh ("merc_drogue", &mesh_dir);
}

void Mercury_Capsule::DeployMain(VESSEL *vessel)
{
	vessel->SetCrossSections (_V(7.5,7.5,70.0));
	vessel->SetCW (0.1, 0.1, 0.1, 1000.0);
	vessel->SetSize (12);
	vessel->ClearMeshes();
	AddCapsule(vessel);
	AddLandingBag(vessel);
	VECTOR3 mesh_dir=OFS_MAINCHUTE;
	vessel->AddMesh ("merc_chute", &mesh_dir);
}


void Mercury_Capsule::SepParachute(VESSEL *vessel)
{
	if (!bMainDeployed && bDrogueDeployed && !bDrogueJettisoned)
	{
		_JettisonDrogue(vessel);
		bDrogueJettisoned=true;
		SetCapsuleMain(vessel);
	}
	else if (bMainDeployed && !bMainJettisoned)
	{
		_JettisonMain(vessel);
		SetCapsuleMain(vessel);
		bMainJettisoned=true;
	}
	else if (bBackupDeployed && !bBackupJettisoned)
	{
		_JettisonMain(vessel);
		SetCapsuleMain(vessel);
		bBackupJettisoned=true;
	}
}



//==============================================================================================================
class Mercury_WithRetro:public IStage
{
public:
	Mercury_WithRetro();
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	bool bManualSeparate;
	bool bRetro;
	bool bDonePosigrade;
	int retro_seq;
	double stage_sep;
	double retro_timer;
	bool bBeaconJettisoned;
	bool bJettisonBeacon;
	int p_seq;
};

Mercury_WithRetro::Mercury_WithRetro()
{
	Init();
};
void Mercury_WithRetro::Init ()
{
	bManualSeparate = false;
	bRetro = false;
	bDonePosigrade = false;
	stage_sep = 0.0;
	retro_timer = 0.0;
	retro_seq = 0;
	p_seq;
	bBeaconJettisoned=false;
	bJettisonBeacon=false;
}

void Mercury_WithRetro::Exit(VESSEL *vessel)
{
}
void Mercury_WithRetro::Setup (VESSEL *vessel, double stage_sep_time)
{
	stage_sep=stage_sep_time;
	SetCapsuleWithRetro(vessel,true);
}

void Mercury_WithRetro::LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{

}

void Mercury_WithRetro::SaveState (VESSEL *vessel, FILEHANDLE scn)
{

}

void Mercury_WithRetro::Timestep (VESSEL *vessel, double simt)
{
	Mercury_Capsule *newStage;
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	if (!bDonePosigrade) 
	{
		vessel->ClearExhaustRefs();
		m_exhaust_pos= _V(.08,.27,OFS_RETROPACK.z-.11-.25);
		m_exhaust_ref = _V(0,0,-1);
		vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, .5, 0.02, &m_exhaust_ref);
		m_exhaust_pos= _V(-.259,-.035,OFS_RETROPACK.z-.11-.25);
		m_exhaust_ref = _V(0,0,-1);
		vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, .5, 0.02, &m_exhaust_ref);
		m_exhaust_pos= _V(.2,-.155,OFS_RETROPACK.z-.11-.25);
		m_exhaust_ref = _V(0,0,-1);
		vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, .5, 0.02, &m_exhaust_ref);
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);
		bDonePosigrade = true;

	}
	if (bRetro)
	{
		if (retro_seq == 0)
		{
			vessel->SetEmptyMass(1355.0-19.0*3);
			vessel->SetEngineLevel(ENGINE_MAIN,0.0);
			vessel->SetMaxFuelMass(19.0*3);
			vessel->SetFuelMass(19.0*3);
			vessel->SetISP(2300.0);
			vessel->SetMaxThrust(ENGINE_MAIN, 4500.0);
			vessel->ClearExhaustRefs();
			m_exhaust_pos= _V(-.07,-.2,OFS_RETROPACK.z-.11-.25);
			m_exhaust_ref = _V(0,0,-1);
			vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.8, 0.06, &m_exhaust_ref);
			retro_seq = 1;
			bRetro = false;
		}
		else if (retro_seq == 1)
		{
			retro_timer = simt;
			vessel->SetEngineLevel(ENGINE_MAIN, 1.0);
			retro_seq = 2;
		}
		else if (retro_seq == 2)
		{
			if (simt-retro_timer >= 15)
			{
				vessel->SetMaxThrust(ENGINE_MAIN, 4500.0*1);
				vessel->ClearExhaustRefs();
				m_exhaust_pos= _V(.19,.05,OFS_RETROPACK.z-.11-.25);
				m_exhaust_ref = _V(0,0,-1);
				vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.8, 0.05, &m_exhaust_ref);
			}
			else if (simt-retro_timer >= 10)
			{
				vessel->SetMaxThrust(ENGINE_MAIN, 4500.0*2);
				vessel->ClearExhaustRefs();
				m_exhaust_pos= _V(.19,.05,OFS_RETROPACK.z-.11-.25);
				m_exhaust_ref = _V(0,0,-1);
				vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.8, 0.05, &m_exhaust_ref);
				m_exhaust_pos= _V(-.14,.14,OFS_RETROPACK.z-.11-.25);
				m_exhaust_ref = _V(0,0,-1);
				vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.8, 0.05, &m_exhaust_ref);
			}
			else if (simt-retro_timer >= 5)
			{
				vessel->SetMaxThrust(ENGINE_MAIN, 4500.0*2);
				vessel->ClearExhaustRefs();
				m_exhaust_pos= _V(-.07,-.2,OFS_RETROPACK.z-.11-.25);
				m_exhaust_ref = _V(0,0,-1);
				vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.8, 0.06, &m_exhaust_ref);
				m_exhaust_pos= _V(-.14,.14,OFS_RETROPACK.z-.11-.25);
				m_exhaust_ref = _V(0,0,-1);
				vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.8, 0.05, &m_exhaust_ref);
			}
			if (vessel->GetFuelMass()==0)
			{
				vessel->SetEngineLevel(ENGINE_MAIN, 0);
				bRetro=false;
				retro_seq = 3;
			}
		}
	}
	if (bJettisonBeacon && !bBeaconJettisoned)
	{
		bBeaconJettisoned=true;
		bJettisonBeacon=false;
		_JettisonBeacon(vessel);
		SetCapsuleWithRetro(vessel,false);
		vessel->SetFuelMass(0.0);
	}
	if (bManualSeparate)
	{
		if (!bBeaconJettisoned)
		{
			_JettisonBeacon(vessel);
		}
		bManualSeparate=false;
		_JettisonRetro(vessel);
		newStage = new Mercury_Capsule();
		Stack->SetStage(newStage,vessel, simt);
	}

}

void Mercury_WithRetro::Animate (VESSEL *vessel, double simt)
{

}

void Mercury_WithRetro::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{

}

void Mercury_WithRetro::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)

{

}

int Mercury_WithRetro::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0;
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
	}
	else 
	{ 
		if (KEYDOWN (keystate, OAPI_KEY_J)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_D)) { // "Retro DeOrbit"
			if (oapiAcceptDelayedKey (OAPI_KEY_D, 1.0))
				bRetro = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_DIVIDE)) { // "Disable Laterals"
			oapiSetAttitudeMode(vessel->GetHandle(),0);
			return 0;
		}
		if (KEYDOWN (keystate, OAPI_KEY_B)) { // "Retro DeOrbit"
			bJettisonBeacon = true;
			return 1;
		}
	}
	return 0;
};



//==============================================================================================================



class Mercury_Abort:public IStage
{
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void AddExhaust(VESSEL *vessel);
	void SetAbortStage(VESSEL *vessel);
	void JettisonAbortTower(VESSEL *vessel);
	int stage;

};

void Mercury_Abort::Init ()
{
	stage = 0;
};


void Mercury_Abort::Animate (VESSEL *vessel, double simt)
{

}

void Mercury_Abort::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{

}

void Mercury_Abort::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)

{

}


void Mercury_Abort::Exit(VESSEL *vessel)
{

}

void Mercury_Abort::LoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{

}

void Mercury_Abort::SaveState (VESSEL *vessel, FILEHANDLE scn)
{

}

void Mercury_Abort::Setup (VESSEL *vessel, double stage_sep_time)
{
	Mercury_Abort::SetAbortStage(vessel);
	vessel->SetEngineLevel(ENGINE_MAIN,1.0);
};

int Mercury_Abort::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0;
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
	}
	else 
	{ 
		if (KEYDOWN (keystate, OAPI_KEY_J)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
//				bManualSeparate = true;
			return 1;
		}
	}
	return 0;
};

void Mercury_Abort::Timestep (VESSEL *vessel, double simt)
{
	Mercury_Capsule *newStage;
	if (stage == 0 && vessel->GetFuelMass() == 0)
	{
		JettisonAbortTower(vessel);
		SetCapsuleWithRetro(vessel,false);
		vessel->SetFuelMass(0);
		stage = 1;
	}

	else if (stage == 1)
	{
		_JettisonRetro(vessel);
		if (vessel->GetAtmPressure() > 66000)
		{
			_JettisonAntenna(vessel);
		}
		vessel->SetFuelMass(0);
		newStage = new Mercury_Capsule();
		Stack->SetStage(newStage,vessel, simt);
		stage = 2;
		return;
	}

};

void Mercury_Abort::SetAbortStage(VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (1.5+1.9);
	vessel->SetCOG_elev (1.5);
	vessel->SetEmptyMass (1355.0+145.0);
	vessel->SetMaxFuelMass (75.0);
	vessel->SetFuelMass(75.0);
	vessel->SetISP (325.0*9.8);
	vessel->SetMaxThrust (ENGINE_MAIN, 232000.0);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
// ==============================================================
// This is from the altanis launch config and is what makes the V-2 launch vertically
// The Z values you use to set the rocket up into the air and align with the height of 
// your launch platform. In this case the Pad 39A.
	vessel->SetTouchdownPoints (_V(0,-1.0,-1.48), _V(-.7,.7,-1.48), _V(.7,.7,-1.48));
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(2.13,2.13,0.52));
	vessel->SetCrossSections (_V(4.25,4.25,3.82));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.1, 0.1, 0.1, MERC_DRAG);
	vessel->SetRotDrag (_V(0.7,0.7,1.2));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
// ==============================================================
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	AddAbortTower(vessel);
	AddAntennaHousing(vessel);
	AddCapsule(vessel);
	AddRetro(vessel);
	AddExhaust(vessel);
};

void Mercury_Abort::AddExhaust(VESSEL *vessel)
{
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref; 
	m_exhaust_pos= _V(0,.300,.20+1.48+1.35);
	m_exhaust_ref = _V(0,.300,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);
	m_exhaust_pos= _V(0,-.300,.20+1.48+1.35);
	m_exhaust_ref = _V(0,-.300,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);
	m_exhaust_pos= _V(.300,0,.20+1.48+1.35);
	m_exhaust_ref = _V(.349,0,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);
	m_exhaust_pos= _V(-.300,0,.20+1.48+1.35);
	m_exhaust_ref = _V(-.349,0,-.946);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 1.0, 0.05, &m_exhaust_ref);
};

void Mercury_Abort::JettisonAbortTower(VESSEL *vessel)
{
	vessel->SetEngineLevel(ENGINE_MAIN,0);
	_JettisonAbortTower(vessel);	
}

#endif // !__MULTI_STAGE_H
