// ==============================================================
// ORBITER vessel module: Atlas Booster
// ==============================================================

#include "orbitersdk.h"
const VECTOR3 OFS_ANTENNA = {0.0, 0.0, 0.0};
const VECTOR3 OFS_DROGUE = { 0.0, 0.0, .24};


// ==============================================================
// API interface
// ==============================================================



// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}


// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	WORD eng1, eng2;
	vessel->SetSize (0.3);
	vessel->SetCOG_elev (6.75);
	vessel->SetEmptyMass (114.0);
	vessel->SetMaxThrust (ENGINE_MAIN, 0);
	vessel->SetMaxThrust (ENGINE_RETRO, 0);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 5e3);
	vessel->SetPMI (_V(1.0,1.0,1.0));
	vessel->SetCrossSections (_V(2.0,2.0,2.0));
	vessel->SetCW (0.1, 0.1, 0.1, 0.1);
	vessel->SetRotDrag (_V(0.7,0.7,0.7));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	mesh_dir=_V(0,0,1);
	eng1 = vessel->AddExhaustRef(EXHAUST_CUSTOM,_V(0,0,.05),.15,.05,&mesh_dir);
	mesh_dir=_V(0,0,-1);
	eng2 = vessel->AddExhaustRef(EXHAUST_CUSTOM,_V(0,0,-.05),.15,.05,&mesh_dir);
	mesh_dir=OFS_ANTENNA;
	vessel->AddMesh ("merc_beacon", &mesh_dir);
	vessel->SetExhaustScales (EXHAUST_CUSTOM, eng2, .15, .05);
	vessel->SetExhaustScales (EXHAUST_CUSTOM, eng2, .15, .05);
	//mesh_dir=OFS_DROGUE;
	//vessel->AddMesh ("merc_drogue", &mesh_dir);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{

}

